=== modified file 'Makefile'
--- Makefile	2012-01-11 08:06:03 +0000
+++ Makefile	2012-01-10 11:16:56 +0000
@@ -1,4 +1,2 @@
 check:
 	python -m unittest grackle.tests.test_client
-
-.PHONY: check

=== added file 'grackle-create-instance'
--- grackle-create-instance	1970-01-01 00:00:00 +0000
+++ grackle-create-instance	2012-01-20 08:26:17 +0000
@@ -0,0 +1,17 @@
+#!/usr/bin/env python
+
+from optparse import OptionParser
+
+from grackle.server.model import create_schema
+
+
+parser = OptionParser(usage="usage: %prog [options] <hostname> <keyspace>")
+parser.add_option(
+    "--clobber", action="store_true", default=False, dest="clobber",
+    help="clear the keyspace before initialising it")
+options, args = parser.parse_args()
+
+if len(args) != 2:
+    parser.error("hostname and keyspace required")
+
+create_schema(args[0], args[1], clobber=options.clobber)

=== modified file 'grackle-put-message'
--- grackle-put-message	2012-01-13 13:52:09 +0000
+++ grackle-put-message	2012-01-10 11:06:36 +0000
@@ -6,10 +6,9 @@
     """An error caused by a user mistake."""
 
 def main(*args):
-    if len(args) != 2:
+    if len(args) != 1:
         raise UserError("Wrong number of arguments supplied.")
-    grackle = client.GrackleClient('localhost', 8123)
-    grackle.put_message(args[0], args[1], sys.stdin)
+    client.put_message(args[0], sys.stdin)
 
 
 if __name__ == '__main__':

=== modified file 'grackle/client.py'
--- grackle/client.py	2012-01-30 18:28:17 +0000
+++ grackle/client.py	2012-01-20 08:26:17 +0000
@@ -1,110 +1,13 @@
 import httplib
-import simplejson
-from urlparse import urlunparse
-from urllib import (
-    quote,
-    urlencode,
-)
-
-
-class UnsupportedOrder(Exception):
-    """Raised when an Unsupported order is requested."""
-
-
-class GrackleClient:
-    """Class for accessing Grackle web service."""
-
-    def __init__(self, host, port):
-        """Constructor.
-
-        :param host: The name of the server.
-        :param port: The port providing Grackle service.
-        """
-        self.host = host
-        self.port = port
-        self.netloc = '%s:%d' % (host, port)
-
-    def archive_url(self, archive_id, query):
-        """Return the URL for an archive
-
-        :param archive_id: The id of the archive to generate the URL for.
-        :param query: The query to use in the URL, as a dict.
-        """
-        path = '/archive/%s' % quote(archive_id)
-        query_string = urlencode(query)
-        return urlunparse(('http', self.netloc, path, '', query_string, ''))
-
-    def _get_connection(self):
-        return httplib.HTTPConnection(self.host, self.port)
-
-    def _method_archive(self, method, archive_id, query, body=None):
-        """Perform an HTTP method on an archive's URL."""
-        url = self.archive_url(archive_id, query)
-        connection = self._get_connection()
-        connection.request(method, url, body)
-        return connection.getresponse()
-
-    def put_message(self, archive_id, key, file_obj):
-        """Put a message into an archive.
-
-        :param archive_id: The archive to put the message into.
-        :param key: An arbitrary identifier that can later be used to retrieve
-            the message.
-        :param file_obj: The raw text of the message, as a file.
-        """
-        response = self._method_archive(
-            'POST', archive_id, {'key': key}, file_obj.read())
-        response.read()
-        if response.status == httplib.BAD_REQUEST:
-            raise Exception('wtf')
-        elif response.status == httplib.CREATED:
-            return
-        else:
-            raise Exception('!!')
-
-    def get_messages(self, archive_id, message_ids=None, limit=None,
-                     memo=None, order=None, headers=None,
-                     max_body_length=None, include_hidden=False):
-        """Retrieve specified messages.
-
-        :param archive_id: The archive to retrieve messages from.
-        :param message_ids: (optional) Retrieve only messages with these ids.
-        :param limit: The maximum number of messages to return.  The server
-            may, at its discretion, return fewer.
-        :param memo: (optional) Opaque identifier describing the position in
-            the list of messages to return.  The combination of a memo and a
-            limit describes a batch of results.  If not specified, the start
-            is used.
-        :param order: The order to return results in.  Supported orders are
-            determined by the server.  See test_client.SUPPORTED_ORDERS for an
-            example.
-        :param headers: The headers to include in the message.  Only headers
-            actually present in the message will be provided.  If unspecified,
-            most headers will be included.
-        :param max_body_length: The maximum length for a message's body.  When
-            multiple messages are nested (as with a thread), this applies to
-            each message's body, not the aggregate length of all messages'
-            bodies.
-        :param include_hidden: If true, include messages that have been
-            flagged "hidden" in the results.
-        """
-        parameters = {}
-        if message_ids is not None:
-            parameters['message_ids'] = message_ids
-        if limit is not None:
-            parameters['limit'] = limit
-        if memo is not None:
-            parameters['memo'] = memo
-        if order is not None:
-            parameters['order'] = order
-        if headers is not None:
-            parameters['headers'] = headers
-        if max_body_length is not None:
-            parameters['max_body_length'] = max_body_length
-        parameters['include_hidden'] = include_hidden
-        query = {'parameters': simplejson.dumps(parameters)}
-        response = self._method_archive('GET', archive_id, query)
-        if response.status == httplib.BAD_REQUEST:
-            raise UnsupportedOrder
-        data = response.read()
-        return simplejson.loads(data)
+
+def put_message(archive_name, file_obj):
+    connection = httplib.HTTPConnection('localhost', 8435)
+    connection.request('POST', '/archive/%s' % archive_name, file_obj.read())
+    response = connection.getresponse()
+    data = response.read()
+    if response.status == httplib.BAD_REQUEST:
+        raise Exception('wtf')
+    elif response.status == httplib.CREATED:
+        return
+    else:
+        raise Exception('!!')

=== added directory 'grackle/server'
=== added file 'grackle/server/__init__.py'
=== added file 'grackle/server/cassandra.py'
--- grackle/server/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/server/cassandra.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Things to ease working with cassandra."""
+
+from pycassa.pool import InvalidRequestError
+
+
+def workaround_1779(callable, *args, **kwargs):
+    """Workaround cassandra not being able to do concurrent schema edits.
+
+    The callable is tried until it does not raised InvalidRequestException
+    with why = "Previous version mismatch. cannot apply."
+
+    :param callable: The callable to call.
+    :param args: The args for it.
+    :param kwargs: The kwargs for it.
+    :return: The result of calling the callable.
+    """
+    while True:
+        # Workaround https://issues.apache.org/jira/browse/CASSANDRA-1779:
+        # Cassandra cannot do concurrent schema changes.
+        try:
+            return callable(*args, **kwargs)
+            break
+        except InvalidRequestError as e:
+            if e.why != 'Previous version mismatch. cannot apply.':
+                raise

=== added file 'grackle/server/model.py'
--- grackle/server/model.py	1970-01-01 00:00:00 +0000
+++ grackle/server/model.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,80 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+import datetime
+import uuid
+
+import pycassa
+from pycassa.system_manager import (
+    LEXICAL_UUID_TYPE,
+    SystemManager,
+    TIME_UUID_TYPE,
+    )
+
+from grackle.server.cassandra import workaround_1779
+
+
+def create_schema(host, keyspace, clobber=False):
+    mgr = SystemManager(host)
+
+    if clobber:
+        for cf in mgr.get_keyspace_column_families(keyspace):
+            mgr.drop_column_family(keyspace, cf)
+
+    try:
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'message',
+            key_validation_class=LEXICAL_UUID_TYPE)
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'archive_message',
+            comparator_type=TIME_UUID_TYPE,
+            default_validation_class=LEXICAL_UUID_TYPE)
+        pass
+    finally:
+        mgr.close()
+
+
+class CassandraConnection(object):
+
+    def __init__(self, keyspace, host):
+        self._keyspace = keyspace
+        self._host = host
+        self._connection = self._connect()
+        self.messages = self._column_family('message')
+        self.archive_messages = self._column_family('archive_message')
+
+    def _connect(self):
+        return pycassa.connect(self._keyspace, self._host)
+
+    def _column_family(self, name):
+        return pycassa.ColumnFamily(self._connection, name)
+
+    def add_message(self, archive_uuid, message):
+        message_uuid = uuid.uuid4()
+        self.messages.insert(
+            message_uuid,
+            {'date_created': datetime.datetime.utcnow().isoformat() + 'Z',
+             'content': message,
+             })
+        self.archive_messages.insert(
+            archive_uuid, {datetime.datetime.now(): message_uuid})
+        return message_uuid
+
+    def get_messages(self, archive_uuid):
+        ids = self.archive_messages.get(
+            archive_uuid, column_count=10, column_reversed=True).values()
+        messages = self.messages.multiget(ids)
+        return [messages[id]['content'] for id in ids]

=== added directory 'grackle/server/testing'
=== added file 'grackle/server/testing/__init__.py'
=== added file 'grackle/server/testing/cassandra.py'
--- grackle/server/testing/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/server/testing/cassandra.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,54 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Test helpers for working with cassandra."""
+
+import os.path
+
+from fixtures import Fixture, TempDir
+from pycassa.system_manager import SystemManager
+
+from grackle.server.cassandra import workaround_1779
+from grackle.server.model import create_schema
+
+
+class TemporaryKeyspace(Fixture):
+    """Create a temporary keyspace.
+
+    The keyspace is named after a tempdir.
+    """
+
+    def setUp(self):
+        super(TemporaryKeyspace, self).setUp()
+        tempdir = self.useFixture(TempDir())
+        self.keyspace = os.path.basename(tempdir.path)
+        self.mgr = SystemManager()
+        workaround_1779(self.mgr.create_keyspace, self.keyspace,
+            replication_factor=1)
+        self.addCleanup(workaround_1779, self.mgr.drop_keyspace,
+            self.keyspace)
+
+
+class TemporaryDB(Fixture):
+    """Create a temporary usable OOPS DB.
+
+    The keyspace for it is at self.keyspace.
+    """
+
+    def setUp(self):
+        super(TemporaryDB, self).setUp()
+        self.keyspace = self.useFixture(TemporaryKeyspace()).keyspace
+        create_schema('localhost', self.keyspace)

=== added directory 'grackle/server/tests'
=== added file 'grackle/server/tests/__init__.py'
=== added file 'grackle/server/tests/test_wsgi.py'
--- grackle/server/tests/test_wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/server/tests/test_wsgi.py	2012-01-12 10:52:19 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+from unittest import TestCase
+
+from webtest import TestApp
+
+from grackle.server.wsgi import app
+
+
+class TestRoot(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service endpoint for Grackle.')
+
+
+class TestArchive(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/archive/foo')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service for archive foo.')

=== added file 'grackle/server/wsgi.py'
--- grackle/server/wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/server/wsgi.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,68 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+import web
+
+from grackle.server.model import CassandraConnection
+
+conn = None
+
+
+class root:
+
+    def GET(self):
+        web.header('Content-Type', 'text/plain')
+        return (
+            'Service endpoint for Grackle.\n\n'
+            'When you want mail that was archived in Grackle, you can get it '
+            'from Grackle')
+
+
+class archive:
+
+    def GET(self, name):
+        web.header('Content-Type', 'text/plain')
+        args = web.input()
+        if not getattr(args, 'op', None):
+            return self.index(name)
+
+        if args.op == 'get_messages':
+            return self.get_messages(name)
+
+    def POST(self, name):
+        data = web.data()
+        conn.add_message(name, data)
+        web.ctx.status = '201 Created'
+
+    def index(self, name):
+        return ('Service for archive %s.' % name)
+
+    def get_messages(self, name):
+        return repr(conn.get_messages(name))
+
+
+urls = (
+    '/', 'root',
+    '/archive/(.+)', 'archive',
+    )
+
+app = web.application(urls, globals(), autoreload=False)
+
+if __name__ == "__main__":
+    conn = CassandraConnection('grackle', 'localhost/9160')
+    app.run()

=== modified file 'grackle/tests/test_client.py'
--- grackle/tests/test_client.py	2012-01-30 18:28:17 +0000
+++ grackle/tests/test_client.py	2012-01-10 14:03:53 +0000
@@ -3,199 +3,37 @@
     BaseHTTPRequestHandler,
     )
 import httplib
-import logging
 import os
 from signal import SIGKILL
-import simplejson
 from StringIO import StringIO
-import sys
 from unittest import TestCase
-from urlparse import urlparse
-from urlparse import parse_qs
 
 from testtools import ExpectedException
 
-from grackle.client import (
-    GrackleClient,
-    UnsupportedOrder,
-    )
-
-
-def threaded_messages(messages):
-    threads = {}
-    count = 0
-    pending = []
-    for message in messages:
-        if message.get('in_reply_to') is None:
-            threads[message['message_id']] = [message]
-            count += 1
-        else:
-            pending.append(message)
-    for message in pending:
-        threads[message['in_reply_to']].append(message)
-    return threads.values()
-
-
-class GrackleStore:
-    """A memory-backed message store."""
-
-    def __init__(self, messages):
-        """Constructor."""
-        self.messages = messages
-
-    def get_messages(self, archive_id, query_string):
-        """Return matching messages.
-
-        :param archive_id: The archive to retrieve from.
-        :param query_string: Contains 'parameters', which is a JSON-format
-            string describing parameters.
-        """
-        query = parse_qs(query_string)
-        parameters = simplejson.loads(query['parameters'][0])
-        order = parameters.get('order')
-        messages = self.messages[archive_id]
-        if order is not None:
-            if order not in SUPPORTED_ORDERS:
-                raise UnsupportedOrder
-            elif order.startswith('thread_'):
-                threaded = threaded_messages(messages)
-                messages = []
-                if order == 'thread_subject':
-                    threaded.sort(key=lambda t: t[0]['subject'])
-                if order == 'thread_oldest':
-                    threaded.sort(key=lambda t: min(m['date'] for m in t))
-                if order == 'thread_newest':
-                    threaded.sort(key=lambda t: max(m['date'] for m in t))
-                for thread in threaded:
-                    messages.extend(thread)
-            else:
-                messages.sort(key=lambda m: m[order])
-        new_messages = []
-        for message in messages:
-            if (not parameters['include_hidden']
-                and message.get('hidden', False)):
-                continue
-
-            if ('message_ids' in parameters
-                and message['message_id'] not in parameters['message_ids']):
-                continue
-            message = dict(message)
-            if 'headers' in parameters:
-                headers = dict(
-                    (k, v) for k, v in message['headers'].iteritems()
-                    if k in parameters['headers'])
-                message['headers'] = headers
-            max_body = parameters.get('max_body_length')
-            if max_body is not None:
-                message['body'] = message['body'][:max_body]
-            new_messages.append(message)
-        messages = new_messages
-        limit = parameters.get('limit', 100)
-        memo = parameters.get('memo')
-        message_id_indices = dict(
-            (m['message_id'], idx) for idx, m in enumerate(messages))
-        if memo is None:
-            start = 0
-        else:
-            start = message_id_indices[memo.encode('rot13')]
-        if start > 0:
-            previous_memo = messages[start - 1]['message_id'].encode('rot13')
-        else:
-            previous_memo = None
-        end = min(start + limit, len(messages))
-        if end < len(messages):
-            next_memo = messages[end]['message_id'].encode('rot13')
-        else:
-            next_memo = None
-        messages = messages[start:end]
-
-        response = {
-            'messages': messages,
-            'next_memo': next_memo,
-            'previous_memo': previous_memo
-            }
-        return response
-
-
-class ForkedFake:
-    """A Grackle service fake, as a ContextManager."""
-
-    def __init__(self, port, messages=None, write_logs=False):
-        """Constructor.
-        :param port: The tcp port to use
-        :param messages: A dict of lists of dicts representing messages.  The
-            outer dict represents the archive, the list represents the list of
-            messages for that archive.
-        :param write_logs: If true, log messages will be written to stdout.
-        """
+from grackle import client
+
+
+class Forked:
+
+    def __init__(self, func_or_method):
+        self.func_or_method = func_or_method
         self.pid = None
-        self.port = port
-        if messages is None:
-            self.messages = {}
-        else:
-            self.messages = messages
-        self.read_end, self.write_end = os.pipe()
-        self.write_logs = write_logs
-
-    @staticmethod
-    def from_client(client, messages=None):
-        """Instantiate a ForkedFake from the client.
-
-        :param port: The client  to provide service for.
-        :param messages: A dict of lists of dicts representing messages.  The
-            outer dict represents the archive, the list represents the list of
-            messages for that archive.
-        """
-        return ForkedFake(client.port, messages)
-
-    def is_ready(self):
-        """Tell the parent process that the server is ready for writes."""
-        os.write(self.write_end, 'asdf')
 
     def __enter__(self):
-        """Run the service.
-
-        Fork and start a server in the child.  Return when the server is ready
-        for use."""
         pid = os.fork()
-        if pid == 0:
-            self.start_server()
-        self.pid = pid
-        os.read(self.read_end, 1)
-        return
+        if pid != 0:
+            self.pid = pid
+            return
+        self.func_or_method()
 
-    def start_server(self):
-        """Start the HTTP server."""
-        service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
-        service.store = GrackleStore(self.messages)
-        for archive_id, messages in service.store.messages.iteritems():
-            for message in messages:
-                message.setdefault('headers', {})
-        self.is_ready()
-        if self.write_logs:
-            logging.basicConfig(
-                stream=sys.stderr, level=logging.INFO)
-        service.serve_forever()
 
     def __exit__(self, exc_type, exc_val, traceback):
         os.kill(self.pid, SIGKILL)
 
 
-SUPPORTED_ORDERS = set(
-    ['date', 'author', 'subject', 'thread_newest', 'thread_oldest',
-     'thread_subject'])
-
-
 class FakeGrackleRequestHandler(BaseHTTPRequestHandler):
-    """A request handler that forwards to server.store."""
-
-    def __init__(self, *args, **kwargs):
-        """Constructor.  Sets up logging."""
-        self.logger = logging.getLogger('http')
-        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)
 
     def do_POST(self):
-        """Create a message on POST."""
         message = self.rfile.read(int(self.headers['content-length']))
         if message == 'This is a message':
             self.send_response(httplib.CREATED)
@@ -204,211 +42,17 @@
         else:
             self.send_error(httplib.BAD_REQUEST)
 
-    def do_GET(self):
-        """Retrieve a list of messages on GET."""
-        scheme, netloc, path, params, query_string, fragments = (
-            urlparse(self.path))
-        parts = path.split('/')
-        if parts[1] == 'archive':
-            try:
-                response = self.server.store.get_messages(
-                    parts[2], query_string)
-                self.send_response(httplib.OK)
-                self.end_headers()
-                self.wfile.write(simplejson.dumps(response))
-            except UnsupportedOrder:
-                self.send_response(httplib.BAD_REQUEST)
-                self.wfile.write('Unsupported order')
-                return
-
-    def log_message(self, format, *args):
-        """Override log_message to use standard Python logging."""
-        message = "%s - - [%s] %s\n" % (
-            self.address_string(), self.log_date_time_string(), format % args)
-        self.logger.info(message)
+
+def run_service():
+    service = HTTPServer(('', 8435), FakeGrackleRequestHandler)
+    service.serve_forever()
+
 
 
 class TestPutMessage(TestCase):
 
     def test_put_message(self):
-        client = GrackleClient('localhost', 8436)
-        with ForkedFake.from_client(client):
-            client.put_message('arch1', 'asdf', StringIO('This is a message'))
+        with Forked(run_service):
+            client.put_message('arch1', StringIO('This is a message'))
             with ExpectedException(Exception, 'wtf'):
-                client.put_message('arch1', 'asdf',
-                    StringIO('This is not a message'))
-
-
-class TestGetMessages(TestCase):
-
-    def assertIDOrder(self, ids, messages):
-        self.assertEqual(ids, [m['message_id'] for m in messages])
-
-    def assertMessageIDs(self, ids, messages):
-        self.assertIDOrder(
-            sorted(ids), sorted(messages, key=lambda m: m['message_id']))
-
-    def test_get_messages(self):
-        client = GrackleClient('localhost', 8435)
-        with ForkedFake.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
-            response = client.get_messages('baz')
-        self.assertEqual(['bar', 'foo'], sorted(m['message_id'] for m in
-            response['messages']))
-        self.assertIs(None, response['next_memo'])
-        self.assertIs(None, response['previous_memo'])
-
-    def test_get_messages_by_id(self):
-        client = GrackleClient('localhost', 8437)
-        with ForkedFake.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
-            response = client.get_messages('baz', message_ids=['foo'])
-        message, = response['messages']
-        self.assertEqual('foo', message['message_id'])
-
-    def test_get_messages_batching(self):
-        client = GrackleClient('localhost', 8438)
-        with ForkedFake.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
-            response = client.get_messages('baz', limit=1)
-            self.assertEqual(1, len(response['messages']))
-            messages = response['messages']
-            response = client.get_messages(
-                'baz', limit=1, memo=response['next_memo'])
-            self.assertEqual(1, len(response['messages']))
-            messages.extend(response['messages'])
-            self.assertMessageIDs(['foo', 'bar'], messages)
-
-    def get_messages_member_order_test(self, key):
-        client = GrackleClient('localhost', 8439)
-        with ForkedFake.from_client(client,
-                {'baz': [{'message_id': 'foo', key: '2011-03-25'},
-                 {'message_id': 'bar', key: '2011-03-24'}]}):
-            response = client.get_messages('baz')
-            self.assertIDOrder(['foo', 'bar'], response['messages'])
-            response = client.get_messages('baz', order=key)
-            self.assertIDOrder(['bar', 'foo'], response['messages'])
-
-    def test_get_messages_date_order(self):
-        self.get_messages_member_order_test('date')
-
-    def test_get_messages_author_order(self):
-        self.get_messages_member_order_test('author')
-
-    def test_get_messages_subject_order(self):
-        self.get_messages_member_order_test('subject')
-
-    def test_get_messages_thread_subject_order(self):
-        client = GrackleClient('localhost', 8439)
-        with ForkedFake.from_client(client, {'baz': [
-            {'message_id': 'bar', 'subject': 'y'},
-            {'message_id': 'qux', 'subject': 'z'},
-            {'message_id': 'foo', 'subject': 'x', 'in_reply_to': 'qux'},
-            ]}):
-            response = client.get_messages('baz')
-            self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
-            response = client.get_messages('baz', order='subject')
-            self.assertIDOrder(['foo', 'bar', 'qux'], response['messages'])
-            response = client.get_messages('baz', order='thread_subject')
-            self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
-
-    def test_get_messages_thread_oldest_order(self):
-        client = GrackleClient('localhost', 8439)
-        with ForkedFake.from_client(client, {'baz': [
-            {'message_id': 'bar', 'date': 'x'},
-            {'message_id': 'qux', 'date': 'z'},
-            {'message_id': 'foo', 'date': 'y', 'in_reply_to': 'qux'},
-            ]}):
-            response = client.get_messages('baz')
-            self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
-            response = client.get_messages('baz', order='date')
-            self.assertIDOrder(['bar', 'foo', 'qux'], response['messages'])
-            response = client.get_messages('baz', order='thread_oldest')
-            self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
-
-    def test_get_messages_thread_newest_order(self):
-        client = GrackleClient('localhost', 8439)
-        with ForkedFake.from_client(client, {'baz': [
-            {'message_id': 'bar', 'date': 'x'},
-            {'message_id': 'qux', 'date': 'w'},
-            {'message_id': 'foo', 'date': 'y', 'in_reply_to': 'bar'},
-            {'message_id': 'baz', 'date': 'z', 'in_reply_to': 'qux'},
-            ]}):
-            response = client.get_messages('baz', order='date')
-            self.assertIDOrder(
-                ['qux', 'bar', 'foo', 'baz'], response['messages'])
-            response = client.get_messages('baz', order='thread_newest')
-            self.assertIDOrder(
-                ['bar', 'foo', 'qux', 'baz'], response['messages'])
-
-    def test_get_messages_unsupported_order(self):
-        client = GrackleClient('localhost', 8439)
-        with ForkedFake.from_client(client,
-                {'baz': [{'message_id': 'foo', 'date': '2011-03-25'},
-                 {'message_id': 'bar', 'date': '2011-03-24'}]}):
-            with ExpectedException(UnsupportedOrder, ''):
-                client.get_messages('baz', order='nonsense')
-
-    def test_get_messages_headers_no_headers(self):
-        client = GrackleClient('localhost', 8440)
-        with ForkedFake.from_client(client,
-            {'baz': [
-                {'message_id': 'foo'}
-            ]}):
-            response = client.get_messages('baz', headers=[
-                'Subject', 'Date', 'X-Launchpad-Message-Rationale'])
-        first_message = response['messages'][0]
-        self.assertEqual('foo', first_message['message_id'])
-        self.assertEqual({}, first_message['headers'])
-
-    def test_get_messages_headers_exclude_headers(self):
-        client = GrackleClient('localhost', 8441)
-        with ForkedFake.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'headers': {'From': 'me'}}
-            ]}):
-            response = client.get_messages('baz', headers=[
-                'Subject', 'Date', 'X-Launchpad-Message-Rationale'])
-        first_message = response['messages'][0]
-        self.assertEqual('foo', first_message['message_id'])
-        self.assertEqual({}, first_message['headers'])
-
-    def test_get_messages_headers_include_headers(self):
-        client = GrackleClient('localhost', 8442)
-        with ForkedFake.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'headers': {'From': 'me', 'To': 'you'}}
-            ]}):
-            response = client.get_messages('baz', headers=[
-                'From', 'To'])
-        first_message = response['messages'][0]
-        self.assertEqual('foo', first_message['message_id'])
-        self.assertEqual({'From': 'me', 'To': 'you'}, first_message['headers'])
-
-    def test_get_messages_max_body_length(self):
-        client = GrackleClient('localhost', 8443)
-        with ForkedFake.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'body': u'abcdefghi'}
-            ]}):
-            response = client.get_messages('baz', max_body_length=3)
-        first_message = response['messages'][0]
-        self.assertEqual('abc', first_message['body'])
-
-    def test_include_hidden(self):
-        client = GrackleClient('localhost', 8444)
-        with ForkedFake.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'hidden': True},
-                {'message_id': 'bar', 'hidden': False}
-            ]}):
-            response = client.get_messages('baz', include_hidden=True)
-            self.assertMessageIDs(['bar', 'foo'], response['messages'])
-            response = client.get_messages('baz', include_hidden=False)
-            self.assertMessageIDs(['bar'], response['messages'])
+                client.put_message('arch1', StringIO('This is not a message'))

